/*******************************************************************************
 * Copyright (C) 2024
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
#pragma once
#ifndef SC_INCLUDE_DYNAMIC_PROCESSES
#define SC_INCLUDE_DYNAMIC_PROCESSES
#endif

#include <ratatoskrUtils/processingElement/ProcessingElementVC.h>
#include <iostream>
#include <map>
#include <string>
#include <vector>

#include "systemc.h"
#include "tlm.h"
#include "ratatoskrUtils/utils/Structures.h"
#include "ratatoskrUtils/utils/GlobalResources.h"
#include "ratatoskrUtils/utils/Report.h"
#include "ratatoskrUtils/traffic/TrafficPool.h"

#include "networkInterface/NetworkInterfaceTlm.h"

#include "ratatoskrUtils/traffic/synthetic/SyntheticPool.h"
#include "ratatoskrUtils/traffic/task/TaskPool.h"
#include "ratatoskrUtils/traffic/netrace/NetracePool.h"

#include "router/router.h"
#include "noc/noc.h"
using namespace tlm;


class NetworkManager : public sc_module{
public:
    SC_HAS_PROCESS(NetworkManager);

    explicit NetworkManager(sc_module_name, std::string);

    ~NetworkManager() override;

private:
    GlobalResources& globalResources = GlobalResources::getInstance();
    //GlobalReport& globalReport = GlobalReport::getInstance(); ////
    Report& rep = Report::getInstance();
    int dbid;
    std::vector<std::unique_ptr<sc_clock>> clocks;
    std::unique_ptr<TrafficPool> tp;
    NocTlm* tlmNoc;
    std::vector<NetworkParticipant*> networkParticipants;
     std::vector<int> idNis;
    std::vector<std::unique_ptr<PacketSignalContainer>> packetSignalContainers;
    //std::vector<std::unique_ptr<Link>> links; ////
    
    void createClocks(); //Create clocks for Ratatoskr PEs

    void createTrafficPool(); //Create the traffic pool with the tasks for PEs

    void createNetworkParticipants(); //Create Network Participants (creates PE and NI, and connects them)
    
    //void createLinks(); //Connects Network Interface to Routers in NoC

    void runNoC(); //Initialize routers and starts traffic pool

    void log_info(std::string msg);
};