/*******************************************************************************
 * Copyright (C) 2024 
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/

// NocTlm.h
// #ifndef NOCTLM_H
// #define NOCTLM_H
#pragma once

#ifndef SC_INCLUDE_DYNAMIC_PROCESSES
#define SC_INCLUDE_DYNAMIC_PROCESSES
#endif

#include <systemc.h>
#include <tlm.h>
#include <tlm_utils/simple_initiator_socket.h>
#include <tlm_utils/simple_target_socket.h>
#include <queue>
#include <iostream>
#include <vector>
#include <string>
#include <map>

#include "utils/configuration.h"
#include "utils/semaphore_manager.h"
#include "router/router.h"
#include "networkInterface/NetworkInterfaceTlm.h"
#include "ratatoskrUtils/utils/Structures.h"
#include "ratatoskrUtils/utils/GlobalReport.h"
#include "ratatoskrUtils/utils/GlobalResources.h"
using namespace tlm;

class NocTlm : public sc_core::sc_module {
public:
  
    SC_HAS_PROCESS(NocTlm);

    // Constructor and destructor
    explicit NocTlm(sc_module_name, std::string);
    ~NocTlm() override;

    uint8_t max_pos[3];
    std::map<int, int> mapNodePERouter;

    void initializeGlobalResources(std::string config_folder);

    GlobalResources& globalResources = GlobalResources::getInstance();

    router* routers[ROUTER_NO];
    NetworkInterfaceTlm* cores[ROUTER_NO][CORE_NO];
    std::vector<sc_core::sc_semaphore*> router_sems;
    std::vector<sc_core::sc_fifo<my_payload*>*> router_out_fifos;
    semaphore_manager* sem_mgr;

    // Method to initialize the routers and cores
    void setup_routers_and_cores();

    void log_info(std::string msg);

    void log_warn(std::string msg);

    void log_error(std::string msg);



};

// #endif // NOCTLM_H
