/*******************************************************************************
 * Copyright (C) 2024
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
#pragma once

#include <systemc.h>
#include <tlm.h>
#include <tlm_utils/simple_initiator_socket.h>
#include <tlm_utils/simple_target_socket.h>
#include <queue>
#include <iostream>
#include <vector>
#include <string>
#include <iostream>
#include <cstdlib>
#include <cstdlib>  // For rand()
#include <ctime>    // For time()

#include "utils/configuration.h"
#include "utils/semaphore_manager.h"

using namespace sc_core;

// Router module with FIFOs, semaphores, and capacity control for receive FIFO
SC_MODULE(router) {
    tlm_utils::simple_target_socket<router> sockets[CORE_NO];  // Sockets to receive data from cores
    tlm_utils::simple_initiator_socket<router> core_sockets[CORE_NO];  // Sockets to send data to cores
    sc_core::sc_fifo<my_payload*> in_fifo;  // FIFO for incoming transactions from cores
    sc_core::sc_fifo<my_payload*> out_fifo;  // FIFO for transactions to be processed by the router
   
    
    unsigned int current_router_id;

    // Declare the semaphore for access control
    sc_core::sc_semaphore sem;
    //int router_id;
    const int D = 10;  // Base delay for each router communication (in ns)
    

    // Pointers to other routers' FIFOs and semaphores for inter-router communication
    std::vector<sc_core::sc_fifo<my_payload*>*> other_out_fifos;
    std::vector<sc_core::sc_semaphore*> other_sems;    

    // Add a pointer to the semaphore manager
    semaphore_manager* sem_mgr;

    SC_CTOR(router);

    // Function to handle incoming transactions
    tlm::tlm_sync_enum nb_transport_fw(tlm::tlm_generic_payload& trans, tlm::tlm_phase& phase, sc_time& delay);

    // Thread to process core_fifo
    void process_core_fifo();
    
    // Thread to process router_receive_fifo and forward transactions to cores
    void process_router_receive_fifo();

    void log_info(std::string msg);
    void log_error(std::string msg);

    // Method to set the semaphore manager instance after creation
    void set_semaphore_manager(semaphore_manager* manager) {
        sem_mgr = manager;
    }
};   

