/*******************************************************************************
 * Copyright (C) 2024
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/

#pragma once

#include <systemc.h>
#include <tlm.h>
#include <tlm_utils/simple_initiator_socket.h>
#include <tlm_utils/simple_target_socket.h>
#include "ratatoskrUtils/utils/Structures.h"
#include <ratatoskrUtils/utils/TrafficTracer.h>
#include <ratatoskrUtils/utils/GlobalResources.h>

using namespace sc_core;  // For sc_time

// Global parameters to generalize the design
#define ROUTER_NO 4 // Number of routers
#define CORE_NO 4  // Number of cores per router
#define CORE_FIFO_CAPACITY 8  // Capacity of FIFO in router that receive data from cores
#define ROUTER_RECEIVE_FIFO_CAPACITY 4 // Capacity of FIFO in router that receive data from other routers
#define LOG_NAME "MY_LOG"
#define NI_LOG           "TLM_NI"


struct my_payload : public tlm::tlm_generic_payload {
    unsigned int src_core;      // Source core ID
    unsigned int src_router;    // Source router ID
    unsigned int dst_core;      // Destination core ID
    unsigned int dst_router;    // Destination router ID
    Flit* data;                   // Data value 
    sc_core::sc_time timestamp;          // Timestamp field

    // Default constructor
    my_payload() : src_core(0), src_router(0), dst_core(0), dst_router(0), data(nullptr), timestamp(SC_ZERO_TIME) {
        set_data_ptr(reinterpret_cast<unsigned char*>(&data));
        set_data_length(sizeof(data));
        set_streaming_width(sizeof(data));
    }

    // Constructor with random data
    my_payload(unsigned int source_core, unsigned int src_router_id, unsigned int dst_core_id, unsigned int destination_router_id, Flit* flit_data, sc_time time = SC_ZERO_TIME)
        : src_core(source_core), src_router(src_router_id), dst_core(dst_core_id), dst_router(destination_router_id), data(flit_data), timestamp(time)  {
        set_data_ptr(reinterpret_cast<unsigned char*>(&data));
        set_data_length(sizeof(data));
        set_streaming_width(sizeof(data));
    }

    // Function to update timestamp at any stage
    void update_timestamp(sc_time new_time) {
    timestamp = new_time;
    }
};


